@@ -33,22 +33,32 @@ using System.Xml.Serialization;
 // To generate the XMLSchema.cs file:
 // > xsd.exe XMLSchema.xsd /classes /l:cs /n:XMLSchema /order
 
+using XSDDiagram.Rendering;
+using System.Xml.Schema;
+using System.Diagnostics;
+
 namespace XSDDiagram
 {
 	public partial class MainForm : Form
 	{
+        private DiagramPrinter _diagramPrinter;
+
         private Diagram diagram = new Diagram();
         private Schema schema = new Schema();
-		private Hashtable hashtableTabPageByFilename = new Hashtable();
+        private Dictionary<string, TabPage> hashtableTabPageByFilename = new Dictionary<string, TabPage>();
 		private string originalTitle = "";
-		private DiagramBase contextualMenuPointedElement = null;
+		private DiagramItem contextualMenuPointedElement = null;
+        private string currentLoadedSchemaFilename = "";
+
+        private TextBox textBoxAnnotation;
+        private WebBrowser webBrowserDocumentation;
+		private bool webBrowserSupported = true;
 
 		public MainForm()
 		{
 			InitializeComponent();
 
 			this.toolsToolStripMenuItem.Visible = !Options.IsRunningOnMono;
-			this.printDialog.UseEXDialog = !Options.IsRunningOnMono;
 
 			this.originalTitle = Text;
 
@@ -63,8 +73,44 @@ namespace XSDDiagram
 			this.panelDiagram.DiagramControl.MouseMove += new MouseEventHandler(DiagramControl_MouseMove);
 			this.panelDiagram.VirtualSize = new Size(0, 0);
 			this.panelDiagram.DiagramControl.Paint += new PaintEventHandler(DiagramControl_Paint);
+
+			if (Options.IsRunningOnMono)
+			{
+				try
+				{
+					new WebBrowser().Navigate("about:blank");
+				}
+				catch
+				{
+					webBrowserSupported = false;
+				}
+			}
 		}
 
+        /// <summary>
+        /// Clean up any resources being used.
+        /// </summary>
+        /// <param name="disposing">true if managed resources should be disposed; otherwise, false.</param>
+        protected override void Dispose(bool disposing)
+        {
+            if (disposing)
+            {
+                if (components != null)
+                {
+                    components.Dispose();
+                    components = null;
+                }
+
+                if (_diagramPrinter != null)
+                {
+                    _diagramPrinter.Dispose();
+                    _diagramPrinter = null;
+                }
+            }
+
+            base.Dispose(disposing);
+        }
+
 		private void MainForm_Load(object sender, EventArgs e)
 		{
 			this.toolStripComboBoxZoom.SelectedIndex = 8;
@@ -129,8 +175,10 @@ namespace XSDDiagram
                 string outputFilename = saveFileDialog.FileName;
 				try
 				{
+                    DiagramExporter exporter = new DiagramExporter(diagram);
                     Graphics g1 = this.panelDiagram.DiagramControl.CreateGraphics();
-                    diagram.SaveToImage(outputFilename, g1, new Diagram.AlerteDelegate(SaveAlert));
+                    exporter.Export(outputFilename, g1, new DiagramAlertHandler(SaveAlert));
+                    g1.Dispose();
 				}
 				catch (Exception ex)
 				{
@@ -145,49 +193,6 @@ namespace XSDDiagram
             return MessageBox.Show(this, message, title, MessageBoxButtons.YesNo) == DialogResult.Yes;
         }
 
-
-		private int currentPage = 0;
-
-		private void printDocument_BeginPrint(object sender, System.Drawing.Printing.PrintEventArgs e)
-		{
-			this.currentPage = 0;
-		}
-
-		private void printDocument_PrintPage(object sender, System.Drawing.Printing.PrintPageEventArgs e)
-		{
-			this.diagram.Layout(e.Graphics);
-
-			Size bbSize = this.diagram.BoundingBox.Size + this.diagram.Padding + this.diagram.Padding;
-			Size totalSize = new Size((int)(bbSize.Width * this.diagram.Scale), (int)(bbSize.Height * this.diagram.Scale));
-
-			int columnNumber = 1 + totalSize.Width / e.MarginBounds.Width;
-			int rowNumber = 1 + totalSize.Height / e.MarginBounds.Height;
-			int pageNumber = columnNumber * rowNumber;
-
-			int row, column = Math.DivRem(currentPage, rowNumber, out row);
-
-			Rectangle clipping = new Rectangle(new Point(column * e.MarginBounds.Width, row * e.MarginBounds.Height),
-				new Size((column + 1) * e.MarginBounds.Width, (row + 1) * e.MarginBounds.Height));
-
-			//MONOFIX e.Graphics.Clip = new Region(e.MarginBounds);
-
-			//Point virtualPoint = this.panelDiagram.VirtualPoint;
-			e.Graphics.TranslateTransform(-(float)(clipping.Left - e.MarginBounds.Left), -(float)(clipping.Top - e.MarginBounds.Top));
-
-			this.diagram.Paint(e.Graphics, clipping);
-
-			if (this.currentPage < pageNumber - 1)
-			{
-				this.currentPage++;
-				e.HasMorePages = true;
-			}
-		}
-
-		private void printDocument_EndPrint(object sender, System.Drawing.Printing.PrintEventArgs e)
-		{
-
-		}
-
 		private void exitToolStripMenuItem_Click(object sender, EventArgs e)
 		{
 			Close();
@@ -235,7 +240,8 @@ namespace XSDDiagram
 			e.Graphics.SmoothingMode = System.Drawing.Drawing2D.SmoothingMode.AntiAlias;
 			e.Graphics.TextRenderingHint = System.Drawing.Text.TextRenderingHint.AntiAlias;
 
-			this.diagram.Paint(e.Graphics, new Rectangle(virtualPoint, this.panelDiagram.DiagramControl.ClientRectangle.Size));
+            DiagramGdiRenderer.Draw(diagram, e.Graphics, 
+                new Rectangle(virtualPoint, this.panelDiagram.DiagramControl.ClientRectangle.Size));
 		}
 
 		private void UpdateDiagram()
@@ -260,11 +266,11 @@ namespace XSDDiagram
 				Text = this.originalTitle;
 		}
 
-		private void LoadSchema(string fileName)
+		private void LoadSchema(string schemaFilename)
 		{
 			Cursor = Cursors.WaitCursor;
 
-			UpdateTitle(fileName);
+			UpdateTitle(schemaFilename);
 
 			this.diagram.Clear();
 			this.panelDiagram.VirtualSize = new Size(0, 0);
@@ -280,7 +286,7 @@ namespace XSDDiagram
 				this.tabControlView.TabPages.RemoveAt(1);
 
 
-            schema.LoadSchema(fileName);
+            schema.LoadSchema(schemaFilename);
 
             foreach (XSDObject xsdObject in schema.Elements)
             {
@@ -308,24 +314,26 @@ namespace XSDDiagram
 			this.tabControlView.SuspendLayout();
 			foreach (string filename in this.schema.XsdFilenames)
 			{
-				WebBrowser webBrowser = new WebBrowser();
-				webBrowser.Dock = DockStyle.Fill;
-				webBrowser.TabIndex = 0;
-
-				string fullPath = filename;
-				try
-				{
-					new Uri(filename);
-				}
-				catch
-				{
-					fullPath = Path.GetFullPath(filename);
-				}
-
+                string fullPath = filename;
+                Control browser = null;
+				if(webBrowserSupported)
+                    browser = new WebBrowser();
+                else
+                    browser = new System.Windows.Forms.TextBox() { Multiline = true, ReadOnly = true, ScrollBars = ScrollBars.Both };
+                browser.Dock = DockStyle.Fill;
+                browser.TabIndex = 0;
+                try
+                {
+                    new Uri(filename);
+                }
+                catch
+                {
+                    fullPath = Path.GetFullPath(filename);
+                }
 				TabPage tabPage = new TabPage(Path.GetFileNameWithoutExtension(filename));
 				tabPage.Tag = fullPath;
 				tabPage.ToolTipText = fullPath;
-				tabPage.Controls.Add(webBrowser);
+                tabPage.Controls.Add(browser);
 				tabPage.UseVisualStyleBackColor = true;
 
 				this.tabControlView.TabPages.Add(tabPage);
@@ -333,6 +341,8 @@ namespace XSDDiagram
 
 			}
 			this.tabControlView.ResumeLayout();
+
+            currentLoadedSchemaFilename = schemaFilename;
 		}
 
 		void DiagramControl_MouseClick(object sender, MouseEventArgs e)
@@ -340,12 +350,12 @@ namespace XSDDiagram
 			Point location = e.Location;
 			location.Offset(this.panelDiagram.VirtualPoint);
 
-			DiagramBase resultElement;
-			DiagramBase.HitTestRegion resultRegion;
+			DiagramItem resultElement;
+			DiagramHitTestRegion resultRegion;
 			this.diagram.HitTest(location, out resultElement, out resultRegion);
-			if (resultRegion != DiagramBase.HitTestRegion.None)
+			if (resultRegion != DiagramHitTestRegion.None)
 			{
-				if (resultRegion == DiagramBase.HitTestRegion.ChildExpandButton)
+				if (resultRegion == DiagramHitTestRegion.ChildExpandButton)
 				{
 					if (resultElement.HasChildElements)
 					{
@@ -361,7 +371,7 @@ namespace XSDDiagram
 						this.panelDiagram.ScrollTo(this.diagram.ScalePoint(resultElement.Location), true);
 					}
 				}
-				else if (resultRegion == DiagramBase.HitTestRegion.Element)
+				else if (resultRegion == DiagramHitTestRegion.Element)
 				{
 					if ((ModifierKeys & (Keys.Control | Keys.Shift)) == (Keys.Control | Keys.Shift)) // For Debug
 					{
@@ -376,7 +386,7 @@ namespace XSDDiagram
 			}
 		}
 
-		private void SelectDiagramElement(DiagramBase element)
+		private void SelectDiagramElement(DiagramItem element)
 		{
 			this.textBoxElementPath.Text = "";
 			if (element == null)
@@ -387,20 +397,23 @@ namespace XSDDiagram
 				return;
 			}
 
-			if (element is DiagramBase)
+			if (element is DiagramItem)
 			{
-				if (this.schema.ElementsByName[element.FullName] != null)
-					this.toolStripComboBoxSchemaElement.SelectedItem = this.schema.ElementsByName[element.FullName];
+                //if (this.schema.ElementsByName[element.FullName] != null)
+                //	this.toolStripComboBoxSchemaElement.SelectedItem = this.schema.ElementsByName[element.FullName];
+                XSDObject xsdObject;
+                if (this.schema.ElementsByName.TryGetValue(element.FullName, out xsdObject) && xsdObject != null)
+                    this.toolStripComboBoxSchemaElement.SelectedItem = xsdObject;
 				else
 					this.toolStripComboBoxSchemaElement.SelectedItem = null;
 
 				SelectSchemaElement(element);
 
 				string path = '/' + element.Name;
-				DiagramBase parentElement = element.Parent;
+				DiagramItem parentElement = element.Parent;
 				while (parentElement != null)
 				{
-					if (parentElement.Type == DiagramBase.TypeEnum.element && !string.IsNullOrEmpty(parentElement.Name))
+					if (parentElement.ItemType == DiagramItemType.element && !string.IsNullOrEmpty(parentElement.Name))
 						path = '/' + parentElement.Name + path;
 					parentElement = parentElement.Parent;
 				}
@@ -418,7 +431,7 @@ namespace XSDDiagram
 			SelectSchemaElement(xsdObject.Tag, xsdObject.NameSpace);
 		}
 
-		private void SelectSchemaElement(DiagramBase diagramBase)
+		private void SelectSchemaElement(DiagramItem diagramBase)
 		{
 			SelectSchemaElement(diagramBase.TabSchema, diagramBase.NameSpace);
 		}
@@ -451,8 +464,10 @@ namespace XSDDiagram
 					}
 					else if (element.type != null)
 					{
-						XSDObject xsdObject = this.schema.ElementsByName[QualifiedNameToFullName("type", element.type)] as XSDObject;
-						if (xsdObject != null)
+                        //XSDObject xsdObject = this.schema.ElementsByName[QualifiedNameToFullName("type", element.type)] as XSDObject;
+                        //if (xsdObject != null)
+                        XSDObject xsdObject;
+                        if (this.schema.ElementsByName.TryGetValue(QualifiedNameToFullName("type", element.type), out xsdObject) && xsdObject != null)
 						{
 							XMLSchema.annotated annotatedElement = xsdObject.Tag as XMLSchema.annotated;
 							if (annotatedElement is XMLSchema.complexType)
@@ -573,9 +588,11 @@ namespace XSDDiagram
 							if (annotatedContent is XMLSchema.extensionType)
 							{
 								XMLSchema.extensionType extensionType = annotatedContent as XMLSchema.extensionType;
-								XSDObject xsdExtensionType = this.schema.ElementsByName[QualifiedNameToFullName("type", extensionType.@base)] as XSDObject;
-								if (xsdExtensionType != null)
-								{
+                                //XSDObject xsdExtensionType = this.schema.ElementsByName[QualifiedNameToFullName("type", extensionType.@base)] as XSDObject;
+                                //if (xsdExtensionType != null)
+                                XSDObject xsdExtensionType;
+                                if (this.schema.ElementsByName.TryGetValue(QualifiedNameToFullName("type", extensionType.@base), out xsdExtensionType) && xsdExtensionType != null)
+                                {
 									XMLSchema.annotated annotatedExtension = xsdExtensionType.Tag as XMLSchema.annotated;
 									if (annotatedExtension != null)
 									{
@@ -601,9 +618,11 @@ namespace XSDDiagram
 							else if (annotatedContent is XMLSchema.restrictionType)
 							{
 								XMLSchema.restrictionType restrictionType = annotatedContent as XMLSchema.restrictionType;
-								XSDObject xsdRestrictionType = this.schema.ElementsByName[QualifiedNameToFullName("type", restrictionType.@base)] as XSDObject;
-								if (xsdRestrictionType != null)
-								{
+                                //XSDObject xsdRestrictionType = this.schema.ElementsByName[QualifiedNameToFullName("type", restrictionType.@base)] as XSDObject;
+                                //if (xsdRestrictionType != null)
+                                XSDObject xsdRestrictionType;
+                                if (this.schema.ElementsByName.TryGetValue(QualifiedNameToFullName("type", restrictionType.@base), out xsdRestrictionType) && xsdRestrictionType != null)
+                                {
 									XMLSchema.annotated annotatedRestriction = xsdRestrictionType.Tag as XMLSchema.annotated;
 									if (annotatedRestriction != null)
 									{
@@ -757,9 +776,11 @@ namespace XSDDiagram
 			{
 				if (attribute.type != null)
 				{
-					XSDObject xsdObject = this.schema.ElementsByName[QualifiedNameToFullName("type", attribute.type)] as XSDObject;
-					if (xsdObject != null)
-					{
+                    //XSDObject xsdObject = this.schema.ElementsByName[QualifiedNameToFullName("type", attribute.type)] as XSDObject;
+                    //if (xsdObject != null)
+                    XSDObject xsdObject;
+                    if (this.schema.ElementsByName.TryGetValue(QualifiedNameToFullName("type", attribute.type), out xsdObject) && xsdObject != null)
+                    {
 						XMLSchema.annotated annotatedElement = xsdObject.Tag as XMLSchema.annotated;
 						if (annotatedElement is XMLSchema.simpleType)
 							ShowEnumerate(annotatedElement as XMLSchema.simpleType);
@@ -781,9 +802,11 @@ namespace XSDDiagram
 				XMLSchema.element element = annotated as XMLSchema.element;
 				if (element != null && element.type != null)
 				{
-					XSDObject xsdObject = this.schema.ElementsByName[QualifiedNameToFullName("type", element.type)] as XSDObject;
-					if (xsdObject != null)
-					{
+                    //XSDObject xsdObject = this.schema.ElementsByName[QualifiedNameToFullName("type", element.type)] as XSDObject;
+                    //if (xsdObject != null)
+                    XSDObject xsdObject;
+                    if (this.schema.ElementsByName.TryGetValue(QualifiedNameToFullName("type", element.type), out xsdObject) && xsdObject != null)
+                    {
 						XMLSchema.annotated annotatedElement = xsdObject.Tag as XMLSchema.annotated;
 						if (annotatedElement is XMLSchema.simpleType)
 							ShowEnumerate(annotatedElement as XMLSchema.simpleType);
@@ -821,11 +844,44 @@ namespace XSDDiagram
 
 		private void ShowDocumentation(XMLSchema.annotation annotation)
 		{
+            if (this.textBoxAnnotation == null)
+            {
+                // 
+                // webBrowserDocumentation
+                // 
+                if(webBrowserSupported)
+                {
+                    this.webBrowserDocumentation = new System.Windows.Forms.WebBrowser();
+                    this.webBrowserDocumentation.Dock = System.Windows.Forms.DockStyle.Fill;
+                    this.webBrowserDocumentation.Location = new System.Drawing.Point(0, 0);
+                    this.webBrowserDocumentation.MinimumSize = new System.Drawing.Size(20, 20);
+                    this.webBrowserDocumentation.Name = "webBrowserDocumentation";
+                    this.webBrowserDocumentation.Size = new System.Drawing.Size(214, 117);
+                    this.webBrowserDocumentation.TabIndex = 1;
+                    this.splitContainerDiagramElement.Panel2.Controls.Add(this.webBrowserDocumentation);
+                }
+                else
+                    this.webBrowserDocumentation = null;
+                // 
+                // textBoxAnnotation
+                // 
+                this.textBoxAnnotation = new System.Windows.Forms.TextBox();
+                this.textBoxAnnotation.Dock = System.Windows.Forms.DockStyle.Fill;
+                this.textBoxAnnotation.Location = new System.Drawing.Point(0, 0);
+                this.textBoxAnnotation.Multiline = true;
+                this.textBoxAnnotation.Name = "textBoxAnnotation";
+                this.textBoxAnnotation.ReadOnly = true;
+                this.textBoxAnnotation.ScrollBars = System.Windows.Forms.ScrollBars.Vertical;
+                this.textBoxAnnotation.Size = new System.Drawing.Size(214, 117);
+                this.textBoxAnnotation.TabIndex = 0;
+                this.splitContainerDiagramElement.Panel2.Controls.Add(this.textBoxAnnotation);
+            }
 			if (annotation == null)
 			{
 				this.textBoxAnnotation.Text = "";
 				this.textBoxAnnotation.Visible = true;
-				this.webBrowserDocumentation.Visible = false;
+                if (this.webBrowserDocumentation != null)
+    				this.webBrowserDocumentation.Visible = false;
 
 				return;
 			}
@@ -853,13 +909,22 @@ namespace XSDDiagram
 						//text = string.Join("\r\n", textLines);
 						this.textBoxAnnotation.Text = text;
 						this.textBoxAnnotation.Visible = true;
-						this.webBrowserDocumentation.Visible = false;
+                        if (this.webBrowserDocumentation != null)
+                            this.webBrowserDocumentation.Visible = false;
 					}
 					else if (documentation.source != null)
 					{
-						this.textBoxAnnotation.Visible = false;
-						this.webBrowserDocumentation.Visible = true;
-						this.webBrowserDocumentation.Navigate(documentation.source);
+                        if (this.webBrowserDocumentation != null)
+                        {
+                            this.textBoxAnnotation.Visible = false;
+                            this.webBrowserDocumentation.Visible = true;
+                            this.webBrowserDocumentation.Navigate(documentation.source);
+                        }
+                        else
+                        {
+                            this.textBoxAnnotation.Text = documentation.source;
+                            this.textBoxAnnotation.Visible = true;
+                        }
 					}
 					break;
 				}
@@ -961,7 +1026,13 @@ namespace XSDDiagram
 		{
 			try
 			{
-				this.pageSetupDialog.ShowDialog(this);
+                if (_diagramPrinter == null)
+                {
+                    _diagramPrinter = new DiagramPrinter();
+                }
+                _diagramPrinter.Diagram = diagram;
+
+                _diagramPrinter.PageSetup();
 			}
 			catch (Exception ex)
 			{
@@ -973,7 +1044,13 @@ namespace XSDDiagram
 		{
 			try
 			{
-				this.printPreviewDialog.ShowDialog(this);
+                if (_diagramPrinter == null)
+                {
+                    _diagramPrinter = new DiagramPrinter();
+                }
+                _diagramPrinter.Diagram = diagram;
+
+                _diagramPrinter.PrintPreview();
 			}
 			catch (Exception ex)
 			{
@@ -985,10 +1062,13 @@ namespace XSDDiagram
 		{
 			try
 			{
-				if (this.printDialog.ShowDialog(this) == DialogResult.OK)
-				{
-					printDocument.Print();
-				}
+                if (_diagramPrinter == null)
+                {
+                    _diagramPrinter = new DiagramPrinter();
+                }
+                _diagramPrinter.Diagram = diagram;
+
+                _diagramPrinter.Print(true, Options.IsRunningOnMono);
 			}
 			catch (Exception ex)
 			{
@@ -1008,15 +1088,15 @@ namespace XSDDiagram
 
 			Point contextualMenuMousePosition = this.panelDiagram.DiagramControl.PointToClient(MousePosition);
 			contextualMenuMousePosition.Offset(this.panelDiagram.VirtualPoint);
-			DiagramBase resultElement;
-			DiagramBase.HitTestRegion resultRegion;
+			DiagramItem resultElement;
+			DiagramHitTestRegion resultRegion;
 			this.diagram.HitTest(contextualMenuMousePosition, out resultElement, out resultRegion);
-			if (resultRegion != DiagramBase.HitTestRegion.None)
+			if (resultRegion != DiagramHitTestRegion.None)
 			{
-				if (resultRegion == DiagramBase.HitTestRegion.Element) // && resultElement.Parent == null)
+				if (resultRegion == DiagramHitTestRegion.Element) // && resultElement.Parent == null)
 				{
 					this.contextualMenuPointedElement = resultElement;
-					this.gotoXSDFileToolStripMenuItem.Enabled = true;
+                    this.gotoXSDFileToolStripMenuItem.Enabled = this.schema.ElementsByName.ContainsKey(this.contextualMenuPointedElement.FullName);
 					this.removeFromDiagramToolStripMenuItem.Enabled = true;
 				}
 			}
@@ -1026,10 +1106,12 @@ namespace XSDDiagram
 		{
 			if (this.contextualMenuPointedElement != null)
 			{
-				XSDObject xsdObject = this.schema.ElementsByName[this.contextualMenuPointedElement.FullName] as XSDObject;
-				if (xsdObject != null)
+				//XSDObject xsdObject = this.schema.ElementsByName[this.contextualMenuPointedElement.FullName] as XSDObject;
+                //if (xsdObject != null)
+				XSDObject xsdObject;
+                if (this.schema.ElementsByName.TryGetValue(this.contextualMenuPointedElement.FullName, out xsdObject) && xsdObject != null)
 				{
-					TabPage tabPage = this.hashtableTabPageByFilename[xsdObject.Filename] as TabPage;
+					TabPage tabPage = this.hashtableTabPageByFilename[xsdObject.Filename];
 					if (tabPage != null)
 						this.tabControlView.SelectedTab = tabPage;
 				}
@@ -1041,7 +1123,7 @@ namespace XSDDiagram
 		{
 			if (this.contextualMenuPointedElement != null)
 			{
-				DiagramBase parentDiagram = this.contextualMenuPointedElement.Parent;
+				DiagramItem parentDiagram = this.contextualMenuPointedElement.Parent;
 				this.diagram.Remove(this.contextualMenuPointedElement);
 				UpdateDiagram();
 				if (parentDiagram != null)
@@ -1057,14 +1139,51 @@ namespace XSDDiagram
 			if (tabControlView.SelectedTab.Tag != null)
 			{
 				WebBrowser webBrowser = tabControlView.SelectedTab.Controls[0] as WebBrowser;
-				if (webBrowser != null)
-				{
-					string url = tabControlView.SelectedTab.Tag as string;
-					//if (webBrowser.Url == null || webBrowser.Url != new Uri(url))
-					if(webBrowser.Document == null)
-						webBrowser.Navigate(url);
-					webBrowser.Select();
-				}
+                if (webBrowser != null)
+                {
+                    string url = tabControlView.SelectedTab.Tag as string;
+                    //if (webBrowser.Url == null || webBrowser.Url != new Uri(url))
+                    if (webBrowser.Document == null)
+                        webBrowser.Navigate(url);
+                    webBrowser.Select();
+                }
+                else
+                {
+                    TextBox textBrowser = tabControlView.SelectedTab.Controls[0] as TextBox;
+                    if (textBrowser != null)
+                    {
+                        string url = tabControlView.SelectedTab.Tag as string;
+                        if (string.IsNullOrEmpty(textBrowser.Text))
+                        {
+                            try
+                            {
+                                //HttpWebRequest webRequestObject = (HttpWebRequest)WebRequest.Create(url);
+                                ////WebRequestObject.UserAgent = ".NET Framework/2.0";
+                                ////WebRequestObject.Referer = "http://www.example.com/";
+                                //WebResponse response = webRequestObject.GetResponse();
+                                //Stream webStream = response.GetResponseStream();
+                                //StreamReader reader = new StreamReader(webStream);
+                                //textBrowser.Text = reader.ReadToEnd();
+                                //reader.Close();
+                                //webStream.Close();
+                                //response.Close();
+
+                                WebClient client = new WebClient();
+                                client.Headers.Add("user-agent", "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.2; .NET CLR 1.0.3705;)");
+                                Stream data = client.OpenRead(url);
+                                StreamReader reader = new StreamReader(data);
+                                textBrowser.Text = reader.ReadToEnd().Replace("\r\n", "\n").Replace("\n", "\r\n");
+                                data.Close();
+                                reader.Close();
+                            }
+                            catch (Exception ex)
+                            {
+                                textBrowser.Text = ex.Message;
+                            }
+                        }
+                        textBrowser.Select();
+                    }
+                }
 			}
 		}
 
@@ -1072,7 +1191,7 @@ namespace XSDDiagram
 		{
 			if (tabControlView.SelectedTab.Tag != null)
 			{
-				WebBrowser webBrowser = tabControlView.SelectedTab.Controls[0] as WebBrowser;
+                Control webBrowser = tabControlView.SelectedTab.Controls[0] as Control;
 				if (webBrowser != null)
 					webBrowser.Select();
 			}
@@ -1082,7 +1201,7 @@ namespace XSDDiagram
 		{
             if (tabControlView.SelectedTab != null && tabControlView.SelectedTab.Tag != null)
 			{
-				WebBrowser webBrowser = tabControlView.SelectedTab.Controls[0] as WebBrowser;
+                Control webBrowser = tabControlView.SelectedTab.Controls[0] as Control;
 				if (webBrowser != null)
 					webBrowser.Focus();
 			}
@@ -1110,14 +1229,14 @@ namespace XSDDiagram
 		{
 			switch (this.toolStripComboBoxAlignement.SelectedItem as string)
 			{
-				case "Top": this.diagram.Alignement = DiagramBase.Alignement.Near; break;
-				case "Center": this.diagram.Alignement = DiagramBase.Alignement.Center; break;
-				case "Bottom": this.diagram.Alignement = DiagramBase.Alignement.Far; break;
+				case "Top": this.diagram.Alignement = DiagramAlignement.Near; break;
+				case "Center": this.diagram.Alignement = DiagramAlignement.Center; break;
+				case "Bottom": this.diagram.Alignement = DiagramAlignement.Far; break;
 			}
 			UpdateDiagram();
 		}
 
-		void diagram_RequestAnyElement(DiagramBase diagramElement, out XMLSchema.element element, out string nameSpace)
+		void diagram_RequestAnyElement(DiagramItem diagramElement, out XMLSchema.element element, out string nameSpace)
 		{
 			element = null;
 			nameSpace = "";
@@ -1409,6 +1528,99 @@ namespace XSDDiagram
 			//e.DrawText();
 		}
 
+        private void validateXMLFileToolStripMenuItem_Click(object sender, EventArgs e)
+        {
+            OpenFileDialog openFileDialog = new OpenFileDialog();
+            openFileDialog.Filter = "xml files (*.xml)|*.xml|All files (*.*)|*.*";
+            openFileDialog.FilterIndex = 2;
+            openFileDialog.RestoreDirectory = true;
+            if (openFileDialog.ShowDialog() == DialogResult.OK)
+            {
+                try
+                {
+                    Cursor = Cursors.WaitCursor;
+
+                    validationErrorMessages.Clear();
+
+                    StreamReader streamReader = new StreamReader(openFileDialog.FileName);
+                    string xmlSource = streamReader.ReadToEnd();
+                    streamReader.Close();
+
+					//XmlDocument x = new XmlDocument();
+					//x.LoadXml(xmlSource);
+
+                    XmlReaderSettings settings = new XmlReaderSettings();
+                    settings.CloseInput = true;
+                    settings.ValidationType = ValidationType.Schema;
+                    settings.ProhibitDtd = false;
+					settings.XmlResolver = null;
+                    settings.ValidationEventHandler += new ValidationEventHandler(ValidationHandler);
+                    settings.ValidationFlags = XmlSchemaValidationFlags.ReportValidationWarnings |
+                                                XmlSchemaValidationFlags.ProcessIdentityConstraints |
+                                                XmlSchemaValidationFlags.ProcessInlineSchema |
+                                                XmlSchemaValidationFlags.ProcessSchemaLocation |
+                                                XmlSchemaValidationFlags.AllowXmlAttributes;
+                    //settings.Schemas.Add("http://www.collada.org/2005/11/COLLADASchema", currentLoadedSchemaFilename);
+                    //settings.Schemas.Add(null, currentLoadedSchemaFilename); // = sc;
+                    List<string> schemas = new List<string>(schema.XsdFilenames);
+                    schemas.Reverse();
+					foreach (string schemaFilename in schemas)
+					{
+						try
+						{
+							settings.Schemas.Add(null, schemaFilename);
+						}
+						catch (Exception ex)
+						{
+							validationErrorMessages.Add(string.Format("Error while parsing {0}, Message: {1}",
+								schemaFilename, ex.Message));
+						}
+					}
+
+                    StringReader r = new StringReader(xmlSource);
+                    using (XmlReader validatingReader = XmlReader.Create(r, settings))
+                    {
+                        while (validatingReader.Read()) { /* just loop through document */ }
+                    }
+
+                    Cursor = Cursors.Default;
+
+                    ErrorReportForm errorReportForm = new ErrorReportForm();
+                    errorReportForm.Errors = validationErrorMessages;
+                    errorReportForm.ShowDialog(this);
+                }
+                catch (Exception ex)
+                {
+					Cursor = Cursors.Default;
+
+					validationErrorMessages.Add(string.Format("Error while validating {0}, Message: {1}",
+						openFileDialog.FileName, ex.Message));
+					//MessageBox.Show("Cannot validate: " + ex.Message);
+					ErrorReportForm errorReportForm = new ErrorReportForm();
+					errorReportForm.Errors = validationErrorMessages;
+					errorReportForm.ShowDialog(this);
+                }
+				Cursor = Cursors.Default;
+
+				if (validationErrorMessages.Count == 0)
+					MessageBox.Show("No issue found");
+			}
+        }
+
+        static List<string> validationErrorMessages = new List<string>();
+
+        public static void ValidationHandler(object sender, ValidationEventArgs e)
+        {
+            //if (e.Severity == XmlSeverityType.Error || e.Severity == XmlSeverityType.Warning)
+            validationErrorMessages.Add(string.Format("{4}: [{3}] Line: {0}, Position: {1} \"{2}\"",
+                e.Exception.LineNumber, e.Exception.LinePosition, e.Exception.Message, validationErrorMessages.Count, e.Severity));
+        }
+
+        private void toolsToolStripMenuItem_DropDownOpening(object sender, EventArgs e)
+        {
+            validateXMLFileToolStripMenuItem.Enabled = (schema != null && schema.XsdFilenames.Count != 0);
+        }
+
 		//void DiagramControl_MouseMove(object sender, MouseEventArgs e)
 		//{
 		//System.Diagnostics.Trace.WriteLine("toolTipDiagramElement_Popup");
